==> xl-utils-2014.9-sources.jar: com/xebialabs/xlplatform/utils/ResourceManagement.scala <==
package com.xebialabs.xlplatform.utils

import java.io.InputStream
import java.io.OutputStream
import scala.language.implicitConversions
  
trait ManagedResource[T] {
  def onEnter(): T
  def onExit(exceptionHappened: Boolean = false): Unit
  def attempt(block: => Unit): Unit = {
    try { block } finally {}
  }
}

class ManagedOutputStream(out:OutputStream) extends ManagedResource[OutputStream] {
  def onEnter(): OutputStream = out
  def onExit(exceptionHappened: Boolean): Unit = {
    attempt(out.close())
  }
}

class ManagedInputStream(in:InputStream) extends ManagedResource[InputStream] {
  def onEnter(): InputStream = in
  def onExit(exceptionHappened: Boolean): Unit = {
    attempt(in.close())
  }
}

object ResourceManagement {
  def map[T <: Any, A <: Any](managed: ManagedResource[T])(body: T => A): A = {
    val resource = managed.onEnter()
    var exception = false
    try {
      body(resource)
    } catch {
      case e: Throwable =>
        exception = true
        managed.onExit(exceptionHappened = true)
        throw e
    } finally {
      if (!exception) managed.onExit()
    }
  }

  def using[T <: Any](managed: ManagedResource[T])(body: T => Unit) {
    map(managed)(body)
  }

  def using[T <: Any, U <: Any](managed1: ManagedResource[T], managed2: ManagedResource[U])(body: T => U => Unit) {
    using[T](managed1) { r: T =>
      using[U](managed2) { s: U => body(r)(s) }
    }
  }

  implicit def os2managed(out:OutputStream): ManagedResource[OutputStream] = new ManagedOutputStream(out)
  implicit def is2managed(in:InputStream): ManagedResource[InputStream] = new ManagedInputStream(in)
}

==> xl-utils-2014.10-sources.jar: com/xebialabs/xlplatform/utils/ResourceManagement.scala <==
package com.xebialabs.xlplatform.utils

import java.io.{Closeable, InputStream, OutputStream}
import scala.language.implicitConversions
  
trait ManagedResource[T] {
  def onEnter(): T
  def onExit(exceptionHappened: Boolean = false): Unit
  def attempt(block: => Unit): Unit = {
    try { block } finally {}
  }
}

class ManagedOutputStream[T <: OutputStream](out: T) extends ManagedResource[T] {
  def onEnter(): T = out
  def onExit(exceptionHappened: Boolean): Unit = {
    attempt(out.close())
  }
}

class ManagedInputStream[T <: InputStream](in: T) extends ManagedResource[T] {
  def onEnter(): T = in
  def onExit(exceptionHappened: Boolean): Unit = {
    attempt(in.close())
  }
}

object ResourceManagement {
  def map[T <: Any, A <: Any](managed: ManagedResource[T])(body: T => A): A = {
    val resource = managed.onEnter()
    var exception = false
    try {
      body(resource)
    } catch {
      case e: Throwable =>
        exception = true
        managed.onExit(exceptionHappened = true)
        throw e
    } finally {
      if (!exception) managed.onExit()
    }
  }

  def using[T <: Any](managed: ManagedResource[T])(body: T => Unit) {
    map(managed)(body)
  }

  def using[T <: Any, U <: Any](managed1: ManagedResource[T], managed2: ManagedResource[U])(body: T => U => Unit) {
    using[T](managed1) { r: T =>
      using[U](managed2) { s: U => body(r)(s) }
    }
  }

  def using[T <: Closeable, U <: Any](closeable: T)(body: T => U): U = {
    try {
      body(closeable)
    } finally {
      closeable.close()
    }
  }

  implicit def os2managed[T <: OutputStream](out: T): ManagedResource[T] = new ManagedOutputStream(out)
  implicit def is2managed[T <: InputStream](in: T): ManagedResource[T] = new ManagedInputStream(in)
}
